@@ -1,12 +1,14 @@
 #' @title
-#' Probability weighted moment.
+#' Probability weighted moments
 #' @description
-#' Calculates probability weighted moments of specific order(s).
-#' @param x vector of data
-#' @param order integer, order of probability weighted moment, can be a set of \{0,1,...\}
-#' @param na.rm logical, indicates if NAs should be removed
-#' @return double, the PWM of orders \code{order} of \code{x}
+#' Calculates empirical probability weighted moments of specific order(s).
+#' @param x numeric vector of data.
+#' @param order integer, order of probability weighted moment, can be a set of \{0,1,...\}.
+#' @param na.rm logical, indicates if NAs should be removed.
+#'
+#' @return numeric vector, empirical PWM of orders \code{order} of \code{x}.
 #' @seealso \code{\link{PWMs}}
+#'
 #' @examples
 #' PWM(rnorm(25))
 #' PWM(rnorm(25), order = 2)
@@ -20,7 +22,7 @@ PWM <- function(x, order = 0, na.rm = FALSE) {
     x <- na.exclude(x)
   }
   r <- vapply(order, function(r) {
-    .Call('TLMoments_pwm_C', PACKAGE = 'TLMoments', x, r)
+    pwm_C(x, r)
   }, numeric(1))
   setNames(r, paste0("beta", order))
 }
